#include "localdef.h"
#include "bstring.h"

ClientInformation *FindSessionKey( std::string &json );

extern "C"
{
/**********************************************************************************************//**
 * \fn  char *Login( char *uid, char *pw, char *cc, char *apik, char *uud, char *sk, char *lud )
 *
 * \brief   Login.
 *          
 *          Verify the user and other parameters.  Details in ApiLogin.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  uid     the User ID.
 * \param [in]  pw      the password.
 * \param [in]  cc      the CustomCode.
 * \param [in]  apik    the apkey.
 * \param [in]  uud     the uuid of the device.
 * \param [in]  sk      the session key.
 * \param [in]  lud     the last updata date of pax, crew, cc lists.
 *
 * \return  Return the results of the logon
 **************************************************************************************************/
__declspec( dllexport ) char *Login( char *uid, 
                                     char *pw, 
                                     char *cc, 
                                     char *apik,
                                     char *uud,
                                     char *sk,
                                     char *lud )
    {
//TRAP_ERROR(( false, "Login - Enter" ));
    static std::string reply;
    std::string fct = "PALogin";

    CheckTimeOuts( reply );
   
    bool result;
    result = ApiLogin( reply, std::string( uid ), std::string( pw ), std::string( cc ), std::string( apik ), std::string( uud ), std::string( sk ), std::string( lud ) );
    if( ! result )
        {
//TRAP_ERROR(( false, "Login - failure" ));        
        SystemShutDown();
        }

//TRAP_ERROR(( false, "Login - success, look for session key" ));
FindSessionKey( sk );
    return( GenerateReply( fct, result, reply, std::string( sk ), std::string( uid ) ) );
    }                                                   

/**********************************************************************************************//**
 * \fn  char *TripList( char *json )
 *
 * \brief   Generate the Trip list.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSON Definitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) BSTR TripList( char *sk )
    {
    BSTR  bs;

//TRAP_ERROR(( false, "TripList - Enter" ));
// char* ch = new char;
//    ch = "{\n""\"PATrips\":\n""\"TripDetails\":\n""[\n""{\n""\"PostFlag\": \"false\",\n""\"TripID\": \"20160504-36FT\",\n""\"StartDate\": 05/04/2016,\n""\"AircraftID\": \"N48751\",\n"" \"LastUpdated\": \"05/19/2016\"\n""\n }\n""{\n""\"PostFlag\": \"true\",\n""\"TripID\": \"20160510-36FT\",\n""\"StartDate\": 05/10/2016,\n""\"AircraftID\": \"N48751\",\n"" \"LastUpdated\": \"05/19/2016\"\n""\n}\n""\n ]},\n";
//    ConvertCharToBSTR( ch, &bs );
//    return( bs );
 
    static std::string reply;
    std::string fct = "PATripList";
//    ClientInformation *ccinfo = FindSessionKey( sk );
//    if( ccinfo == NULL )
//        {
//        reply = "Unknown Session Key ";
//        reply += sk;
//        return( GenerateReply( fct, false, reply, std::string( sk ), std::string( "" ) ) );
//        }

//    std::string user = ccinfo->userid;;

//    CheckTimeOuts( reply );
//TRAP_ERROR(( false, "Test" ));

    ApiTripList( reply, sk );
    
    ConvertCharToBSTR( (Char*)(reply.c_str()), &bs );
//    Char *x = new Char;
    //x = (Char*)(reply.c_str());
    return( bs );
    }

/**********************************************************************************************//**
 * \fn  char *TripCleanup( char *json )
 *
 * \brief   Trip cleanup.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *TripCleanup( const char *sk, const Char *json )
    {
    std::string fct = "PATripCleanup";

    std::string reply;
    ClientInformation *ccinfo = FindSessionKey( sk );

    if( ccinfo == NULL )
        {
        reply = "Unknown Session Key ";
        reply += sk;
        return( GenerateReply( fct, false, reply, std::string( sk ), std::string( "" ) ) );
        }

    CheckTimeOuts( reply );

    std::string user = ccinfo->userid;;

    bool result = ApiTripCleanup( reply, sk, json );
        
    return( GenerateReply( fct, result, reply, std::string( sk ), user ) );
    }

/**********************************************************************************************//**
 * \fn  char *GetTrip( char *json )
 *
 * \brief   Gets a trip.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *GetTrip( char *sk, char *tid )
    {
    std::string reply;
    std::string fct = "PAGetTrip";
    ClientInformation *ccinfo = FindSessionKey( sk );
    if( ccinfo == NULL )
        {
        reply = "Unknown Session Key ";
        reply += sk;
        return( GenerateReply( fct, false, reply, std::string( sk ), std::string( "" ) ) );
        }

    std::string user = ccinfo->userid;;
    CheckTimeOuts( reply );

    if( !ApiGetTrip( reply, sk, tid ) )
        {
        return( GenerateReply( fct, false, reply, std::string( sk ), std::string( "" ) ) );
        }
    
    return( (Char*)(reply.c_str()) );
    }

/**********************************************************************************************//**
 * \fn  char *CostsList( char *json )
 *
 * \brief   Costs list to FT database.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return    json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *CostsList( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiCostsList( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *PutLeg( char *json )
 *
 * \brief   Puts leg to the FT database.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return    json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *PutLeg( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiPutLeg( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *PutPax( char *json )
 *
 * \brief   Puts a pax list.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *PutPax( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiPutPax( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *PutACCost( char *json )
 *
 * \brief   Puts an aircraft cost list.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *PutACCost( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiPutACCost( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *PutCrew( char *json )
 *
 * \brief   Puts a crew list.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return   json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *PutCrew( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiPutCrew( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *PutCrewCost( char *json )
 *
 * \brief   API put crew cost.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *PutCrewCost( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiPutCrewCost( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *UpdateMasterList( char *json )
 *
 * \brief   Update the master lists.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *UpdMasterLists( char *sk )
    {
    std::string reply;

    CheckTimeOuts( reply );

    if( !ApiUpdMasterLists( reply, sk ) )
        {
        SystemShutDown();
        return( "Failure" );
        }
    SystemShutDown();
    return( "Success" );
    }

/**********************************************************************************************//**
 * \fn  char *LogOff( char *json )
 *
 * \brief   Log off.
 *
 * \author  Steven Kibler
 * \date    5/15/2016
 *
 * \param [in]  json as defined in JSONDefinitions_V7.txt
 *
 * \return  json as defined in JSONDefinitions_V7.txt
 **************************************************************************************************/
__declspec( dllexport ) char *LogOff( char *sk )
    {
//TRAP_ERROR(( false, "LogOff - Enter" ));
    std::string fct = "PALogOff";
    std::string reply;

    CheckTimeOuts( reply );

    ClientInformation *ccinfo = FindSessionKey( sk );
//TRAP_ERROR(( false, "Looking for session key %s, result = 0x%8.8lx", sk, ccinfo ));
    std::string user;
    if( ccinfo != NULL )
        {
        user = ccinfo->userid;;
        }

    bool result = false;
    if( (ccinfo == NULL) || !( result = ApiLogOff( reply, sk )) )
        {
        SystemShutDown();
//TRAP_ERROR(( false, "In LogOff Fail exit" ));
        return( GenerateReply( fct, result, reply, std::string( sk ), user ) );
        }
//TRAP_ERROR(( false, "In LogOff Success exit" ));
    SystemShutDown();
//TRAP_ERROR(( false, "In LogOff Success exit" ));
    return( GenerateReply( fct, result, reply, std::string( sk ), user ) );
    }
};
